package com.example.separation.hakudatsu;

import java.util.List;

import com.example.separation.hakudatsu.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.pm.ApplicationInfo;
import android.content.pm.PermissionInfo;
import android.content.pm.ResolveInfo;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

public class HakudatsuSampleActivity extends Activity {
	private static final int DIALOG_PLUGIN_INSTALL = 0;
	private final static int REQUEST_CODE_IMPORT = 1;
	private final static String ACCESS_PERMISSION = "com.example.separation.contactimport.access";
	private static final String TAG = "HakudatsuSample";

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
		
        Button button1 = (Button)findViewById(R.id.button1);
        
		button1.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				PackageManager pm = getPackageManager();

				// プラグイン呼び出しに関連する権限のチェック
				// アクセス制限用permissionが想定通り定義されてるかのチェック
				PermissionInfo pi = null;
				ApplicationInfo applicationInfo = null;
				try {
					pi = pm.getPermissionInfo(ACCESS_PERMISSION, PackageManager.GET_META_DATA);
					applicationInfo = pm.getPackageInfo(pi.packageName, 0).applicationInfo;
				} catch (NameNotFoundException e) {
					// インストール順次第で権限が消える事もある
				}

				// 権限がgrantされている事のチェック
				if (pm.checkPermission(ACCESS_PERMISSION, getPackageName()) == PackageManager.PERMISSION_DENIED) 
				{
					// 権限が未定義、かつ権限がgrantされた後でも無い、となると再インストールしてもらうしか手が無い。orz
					letsReInstall();
					return;
				}
				
				if (pi != null) { // 権限が現在も存在していてるなら内容チェック
					// 権限がアプリ
					String applicationName = "";
					if (applicationInfo != null) {
						applicationName = applicationInfo.loadLabel(pm).toString();
					}
					boolean badSignature = false;
					if (pi.protectionLevel != PermissionInfo.PROTECTION_SIGNATURE) {
						badSignature = true;
					} else {
						badSignature = !(pm.checkSignatures(getPackageName(),
								pi.packageName) == PackageManager.SIGNATURE_MATCH); // signatureだったら宣言したアプリと署名同じだよな？ってチェック
					}
					if (badSignature) {
						String msg = "いいえ。私の戦場はここじゃ無い。 （訳：アプリケーション「"
							+ applicationName
							+ "」("
							+ pi.packageName
							+ ")が悪意ある第三者アプリのようです。アンインストールを推奨します。アンインストールされない限り当機能は動作しません。" 
							+ "しかもこの後当アプリの再インストール作業が待っています…。頑張ろう！）";
						Toast.makeText(HakudatsuSampleActivity.this, msg,
								Toast.LENGTH_LONG).show(); // 第三者アプリが同じ名前でprotectionLevelを変更した権限を勝手に定義してるよー
						waiting(5);
						showUninstallButton(pi.packageName);
						return;
					}
				} else {
					// 権限の定義必須ってのなら、ここでエラー処理やら色々しないとダメよー
				}

				
				// プラグイン呼び出し
				Intent intent = new Intent();
				intent.setClassName("com.example.separation.contactimport", "com.example.separation.contactimport.ContactImportPickActivity");
				List<ResolveInfo> list = pm.queryIntentActivities(intent,
						PackageManager.MATCH_DEFAULT_ONLY);
				if (list.size() > 0) {
					try {
						startActivityForResult(intent, REQUEST_CODE_IMPORT);
					} catch (SecurityException se) {
						// (Android 2.2以降にて) 『
						// このアプリが起動→権限定義してたアプリがアンインストール(署名同じ)される→同じ権限定義している別アプリ(署名違う)がインストール
						// 』という流れになれば理論上起動時チェックを抜けていてもここで権限剥奪されてる可能性アリ
						String msg = se.getMessage();
						Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();
						// アンインストールやら色々の確認はもう一回ボタン押す時に表示するさー（手抜き）
					}
				} else {
					//please install
					showDialog(DIALOG_PLUGIN_INSTALL);
				}
				
			}
		});

    }

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == REQUEST_CODE_IMPORT) {
			switch (resultCode){
			case RESULT_OK:
				String importdata = data.getExtras().getString("importdata");
				Toast.makeText(this, importdata, Toast.LENGTH_LONG).show();
				break;
			case RESULT_CANCELED:
				break;
			}
		}
	}

	@Override
	protected Dialog onCreateDialog(int id) {
		Dialog dialog = null;
		switch (id) {
		case DIALOG_PLUGIN_INSTALL:
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			String message = "連絡先情報から電話番号をぶっこ抜くには取込プラグインが必要です。プラグインをインストールしますか？"
					+ "\n"
					+ "（プラグインは必要な時だけインストールされていれば良いので、要らなくなったら自由にアンインストール出来ます）";
			builder.setMessage(message);
			builder.setPositiveButton("Yes",
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							Uri uri = Uri.parse("market://details?id=com.example.separation.contactimport");
							Intent intent = new Intent();
							intent.setAction(Intent.ACTION_VIEW);
							intent.setData(uri);
							PackageManager pm = getPackageManager();
							List<ResolveInfo> list = pm.queryIntentActivities(intent,
									PackageManager.MATCH_DEFAULT_ONLY);
							if (list.size() > 0) {
								startActivity(intent);
							} else {
								// エミュ環境とかマーケットアプリが無いと対応してるIntentが無いって事。無理に起動すれば例外出る
								Toast.makeText(getApplicationContext(), "多分マーケットアプリ(Google Play)が無いんじゃね？", Toast.LENGTH_LONG).show();
							}
						}
					});
			builder.setNegativeButton("No",
					new DialogInterface.OnClickListener() {

						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.cancel();
						}
					});
			dialog = builder.create();
			break;
		default:
			dialog = null;
			break;
		}
		return dialog;
	}

	/**
	 * http://hascha.blogspot.jp/2012/04/blog-post.html
	 * 
	 * @param packageName
	 */
	private void showUninstallButton(String packageName) {
		Intent intent = new Intent();
		if (Build.VERSION.SDK_INT > 8) {
			intent.setAction("android.settings.APPLICATION_DETAILS_SETTINGS");
			Uri uri = Uri.fromParts("package", packageName, null);
			intent.setData(uri);
		} else {
			String appPkgName;
			if (Build.VERSION.SDK_INT == 8) {
				appPkgName = "pkg";
			} else {
				appPkgName = "com.android.settings.ApplicationPkgName";
			}
			intent.setAction(Intent.ACTION_VIEW);
			intent.setClassName("com.android.settings", "com.android.settings.InstalledAppDetails");
			intent.putExtra(appPkgName, packageName);
		}
		startActivity(intent);
	}

	private void letsReInstall() {
		// 本格的なアプリならアンインストール前にアプリデータをエクスポートしてあとでインポート出来る作りにすべきだよ！
		String msg = "申し訳ないけど僕を一度アンインストールして、再インストールしてくれるかい。";
		Toast.makeText(this, msg, Toast.LENGTH_LONG).show();
		waiting(5);
		showUninstallButton(this.getPackageName());
	}

	private void waiting(long time) {
		try {
			Thread.sleep(time);
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		}
	}

}